@@ -125,6 +125,7 @@ void BarSettingsRead (BarSettings_t *settings) {
 	settings->autoselect = true;
 	settings->history = 5;
 	settings->volume = 0;
+	settings->maxPlayerErrors = 5;
 	settings->sortOrder = BAR_SORT_NAME_AZ;
 	settings->loveIcon = strdup (" <3");
 	settings->banIcon = strdup (" </3");
@@ -243,6 +244,8 @@ void BarSettingsRead (BarSettings_t *settings) {
 				settings->eventCmd = strdup (val);
 			} else if (streq ("history", key)) {
 				settings->history = atoi (val);
+			} else if (streq ("max_player_errors", key)) {
+				settings->maxPlayerErrors = atoi (val);
 			} else if (streq ("sort", key)) {
 				size_t i;
 				static const char *mapping[] = {"name_az",
