@@ -57,8 +57,11 @@ typedef enum {
 	BAR_KS_MANAGESTATION = 23,
 	BAR_KS_PLAYPAUSE2 = 24,
 	BAR_KS_CREATESTATIONFROMSONG = 25,
+	BAR_KS_PLAY = 26,
+	BAR_KS_PAUSE = 27,
+	BAR_KS_VOLRESET = 28,
 	/* insert new shortcuts _before_ this element and increase its value */
-	BAR_KS_COUNT = 26,
+	BAR_KS_COUNT = 29,
 } BarKeyShortcutId_t;
 
 #define BAR_KS_DISABLED '\x00'
@@ -82,7 +85,7 @@ typedef struct {
 
 typedef struct {
 	bool autoselect;
-	unsigned int history;
+	unsigned int history, maxPlayerErrors;
 	int volume;
 	BarStationSorting_t sortOrder;
 	PianoAudioQuality_t audioQuality;
